import { ESLintUtils } from '@typescript-eslint/utils';

/**
 * Rule severity.
 * @since 0.0.1
 */
type Severity = "error" | "off" | "warn";
/**
 * Rule declaration.
 * @internal
 * @since 0.0.1
 */
type RuleDeclaration = [Severity, Record<string, unknown>?] | Severity;
/**
 * Rule config preset.
 * @since 0.0.1
 */
type RulePreset = Record<string, RuleDeclaration>;
/**
 * Rule creator function.
 * @since 0.0.1
 */
type CreateRule = Parameters<ReturnType<typeof ESLintUtils.RuleCreator>>[0]["create"];
/**
 * Rule context.
 * @since 0.0.1
 */
type RuleContext = Parameters<CreateRule>[0];
/**
 * Rule options.
 * @since 0.0.1
 */
type RuleOptions = Parameters<CreateRule>[1];

type RuleCategory = "complexity" | "convention" | "correctness" | "debug" | "pedantic" | "perf" | "restriction" | "security" | "style" | "suspicious";
type RuleCategoryEmoji = "⛔" | "✔️" | "🎨" | "🐞" | "📖" | "🔒" | "🤔" | "🤯" | "🧐" | "🚀";

type RuleNamespace = "debug" | "hooks-extra" | "naming-convention" | "x";

export type { CreateRule, RuleCategory, RuleCategoryEmoji, RuleContext, RuleDeclaration, RuleNamespace, RuleOptions, RulePreset, Severity };
